@@ -2,7 +2,7 @@
 
 /* Background support.
 
-   Copyright (C) 1996-2018
+   Copyright (C) 1996-2025
    Free Software Foundation, Inc.
 
    Written by:
@@ -49,8 +49,7 @@
 #include "lib/tty/key.h"        /* add_select_channel(), delete_select_channel() */
 #include "lib/widget.h"         /* message() */
 #include "lib/event-types.h"
-
-#include "filemanager/fileopctx.h"      /* file_op_context_t */
+#include "lib/util.h"           /* my_fork() */
 
 #include "background.h"
 
@@ -68,6 +67,10 @@ enum ReturnType
     Return_Integer
 };
 
+/*** forward declarations (file scope functions) *************************************************/
+
+static int background_attention (int fd, void *closure);
+
 /*** file scope variables ************************************************************************/
 
 /* File descriptor for talking to our parent */
@@ -78,13 +81,12 @@ static int from_parent_fd;
 
 TaskList *task_list = NULL;
 
-static int background_attention (int fd, void *closure);
-
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-register_task_running (file_op_context_t * ctx, pid_t pid, int fd, int to_child, char *info)
+register_task_running (file_op_context_t *ctx, pid_t pid, int fd, int to_child, char *info)
 {
     TaskList *new;
 
@@ -103,29 +105,31 @@ register_task_running (file_op_context_t * ctx, pid_t pid, int fd, int to_child,
 /* --------------------------------------------------------------------------------------------- */
 
 static int
-destroy_task_and_return_fd (pid_t pid)
+destroy_task (pid_t pid)
 {
     TaskList *p = task_list;
-    TaskList *prev = 0;
+    TaskList *prev = NULL;
 
-    while (p)
+    while (p != NULL)
     {
         if (p->pid == pid)
         {
-            if (prev)
+            int fd = p->fd;
+
+            if (prev != NULL)
                 prev->next = p->next;
             else
                 task_list = p->next;
             g_free (p->info);
             g_free (p);
-            return p->fd;
+            return fd;
         }
         prev = p;
         p = p->next;
     }
 
     /* pid not found */
-    return -1;
+    return (-1);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -222,7 +226,8 @@ background_attention (int fd, void *closure)
     if (bytes == -1 || (size_t) bytes < (sizeof (routine)))
     {
         unregister_task_running (ctx->pid, fd);
-        if (!waitpid (ctx->pid, &status, WNOHANG))
+
+        if (waitpid (ctx->pid, &status, WNOHANG) == 0)
         {
             /* the process is still running, but it misbehaves - kill it */
             kill (ctx->pid, SIGTERM);
@@ -239,64 +244,52 @@ background_attention (int fd, void *closure)
         return 0;
     }
 
-    if ((read (fd, &argc, sizeof (argc)) != sizeof (argc)) ||
-        (read (fd, &type, sizeof (type)) != sizeof (type)) ||
-        (read (fd, &have_ctx, sizeof (have_ctx)) != sizeof (have_ctx)))
-    {
+    if (read (fd, &argc, sizeof (argc)) != sizeof (argc) ||
+        read (fd, &type, sizeof (type)) != sizeof (type) ||
+        read (fd, &have_ctx, sizeof (have_ctx)) != sizeof (have_ctx))
         return reading_failed (-1, data);
-    }
 
     if (argc > MAXCALLARGS)
-    {
         message (D_ERROR, _("Background protocol error"), "%s",
                  _("Background process sent us a request for more arguments\n"
                    "than we can handle."));
-    }
 
-    if (have_ctx)
-    {
-        if (read (fd, ctx, sizeof (*ctx)) != sizeof (*ctx))
-        {
-            return reading_failed (-1, data);
-        }
-    }
+    if (have_ctx != 0 && read (fd, ctx, sizeof (*ctx)) != sizeof (*ctx))
+        return reading_failed (-1, data);
 
     for (i = 0; i < argc; i++)
     {
         int size;
 
         if (read (fd, &size, sizeof (size)) != sizeof (size))
-        {
             return reading_failed (i - 1, data);
-        }
+
         data[i] = g_malloc (size + 1);
+
         if (read (fd, data[i], size) != size)
-        {
             return reading_failed (i, data);
-        }
-        data[i][size] = 0;      /* NULL terminate the blocks (they could be strings) */
+
+        data[i][size] = '\0';   /* NULL terminate the blocks (they could be strings) */
     }
 
     /* Find child task info by descriptor */
     /* Find before call, because process can destroy self after */
-    for (p = task_list; p; p = p->next)
-    {
+    for (p = task_list; p != NULL; p = p->next)
         if (p->fd == fd)
             break;
-    }
 
-    if (p)
+    if (p != NULL)
         to_child_fd = p->to_child_fd;
 
     if (to_child_fd == -1)
         message (D_ERROR, background_process_error, "%s", _("Unknown error in child"));
-
-    /* Handle the call */
-    if (type == Return_Integer)
+    else if (type == Return_Integer)
     {
+        /* Handle the call */
+
         int result = 0;
 
-        if (!have_ctx)
+        if (have_ctx == 0)
             switch (argc)
             {
             case 0:
@@ -342,7 +335,7 @@ background_attention (int fd, void *closure)
 
         /* Send the result code and the value for shared variables */
         ret = write (to_child_fd, &result, sizeof (result));
-        if (have_ctx && to_child_fd != -1)
+        if (have_ctx != 0 && to_child_fd != -1)
             ret = write (to_child_fd, ctx, sizeof (*ctx));
     }
     else if (type == Return_String)
@@ -373,7 +366,8 @@ background_attention (int fd, void *closure)
         default:
             g_assert_not_reached ();
         }
-        if (resstr)
+
+        if (resstr != NULL)
         {
             len = strlen (resstr);
             ret = write (to_child_fd, &len, sizeof (len));
@@ -387,15 +381,17 @@ background_attention (int fd, void *closure)
             ret = write (to_child_fd, &len, sizeof (len));
         }
     }
+
     for (i = 0; i < argc; i++)
         g_free (data[i]);
 
     repaint_screen ();
+
     (void) ret;
+
     return 0;
 }
 
-
 /* --------------------------------------------------------------------------------------------- */
 /* }}} */
 
@@ -407,20 +403,20 @@ background_attention (int fd, void *closure)
  */
 
 static void
-parent_call_header (void *routine, int argc, enum ReturnType type, file_op_context_t * ctx)
+parent_call_header (void *routine, int argc, enum ReturnType type, file_op_context_t *ctx)
 {
     int have_ctx;
     ssize_t ret;
 
-    have_ctx = (ctx != NULL);
+    have_ctx = ctx != NULL ? 1 : 0;
 
     ret = write (parent_fd, &routine, sizeof (routine));
     ret = write (parent_fd, &argc, sizeof (argc));
     ret = write (parent_fd, &type, sizeof (type));
     ret = write (parent_fd, &have_ctx, sizeof (have_ctx));
-
-    if (have_ctx)
+    if (have_ctx != 0)
         ret = write (parent_fd, ctx, sizeof (*ctx));
+
     (void) ret;
 }
 
@@ -446,10 +442,11 @@ parent_va_call (void *routine, gpointer data, int argc, va_list ap)
     }
 
     ret = read (from_parent_fd, &i, sizeof (i));
-    if (ctx)
+    if (ctx != NULL)
         ret = read (from_parent_fd, ctx, sizeof (*ctx));
 
     (void) ret;
+
     return i;
 }
 
@@ -469,24 +466,21 @@ parent_va_call_string (void *routine, int argc, va_list ap)
 
         len = va_arg (ap, int);
         value = va_arg (ap, void *);
-        if ((write (parent_fd, &len, sizeof (len)) != sizeof (len)) ||
-            (write (parent_fd, value, len) != len))
-        {
+        if (write (parent_fd, &len, sizeof (len)) != sizeof (len) ||
+            write (parent_fd, value, len) != len)
             return NULL;
-        }
     }
 
-    if (read (from_parent_fd, &i, sizeof (i)) != sizeof (i))
-        return NULL;
-    if (!i)
+    if (read (from_parent_fd, &i, sizeof (i)) != sizeof (i) || i == 0)
         return NULL;
+
     str = g_malloc (i + 1);
     if (read (from_parent_fd, str, i) != i)
     {
         g_free (str);
         return NULL;
     }
-    str[i] = 0;
+    str[i] = '\0';
     return str;
 }
 
@@ -497,7 +491,7 @@ parent_va_call_string (void *routine, int argc, va_list ap)
 void
 unregister_task_running (pid_t pid, int fd)
 {
-    destroy_task_and_return_fd (pid);
+    destroy_task (pid);
     delete_select_channel (fd);
 }
 
@@ -506,12 +500,13 @@ unregister_task_running (pid_t pid, int fd)
 void
 unregister_task_with_pid (pid_t pid)
 {
-    int fd = destroy_task_and_return_fd (pid);
+    int fd;
+
+    fd = destroy_task (pid);
     if (fd != -1)
         delete_select_channel (fd);
 }
 
-
 /* --------------------------------------------------------------------------------------------- */
 /**
  * Try to make the Midnight Commander a background job
@@ -522,19 +517,24 @@ unregister_task_with_pid (pid_t pid)
  * -1 on failure
  */
 int
-do_background (file_op_context_t * ctx, char *info)
+do_background (file_op_context_t *ctx, char *info)
 {
     int comm[2];                /* control connection stream */
     int back_comm[2];           /* back connection */
     pid_t pid;
 
     if (pipe (comm) == -1)
-        return -1;
+        return (-1);
 
     if (pipe (back_comm) == -1)
-        return -1;
+    {
+        (void) close (comm[0]);
+        (void) close (comm[1]);
 
-    pid = fork ();
+        return (-1);
+    }
+
+    pid = my_fork ();
     if (pid == -1)
     {
         int saved_errno = errno;
@@ -544,14 +544,18 @@ do_background (file_op_context_t * ctx, char *info)
         (void) close (back_comm[0]);
         (void) close (back_comm[1]);
         errno = saved_errno;
-        return -1;
+
+        return (-1);
     }
 
     if (pid == 0)
     {
         int nullfd;
 
+        (void) close (comm[0]);
         parent_fd = comm[1];
+
+        (void) close (back_comm[1]);
         from_parent_fd = back_comm[0];
 
         mc_global.we_are_background = TRUE;
@@ -571,12 +575,15 @@ do_background (file_op_context_t * ctx, char *info)
                 ;
             while (dup2 (nullfd, STDERR_FILENO) == -1 && errno == EINTR)
                 ;
+            close (nullfd);
         }
 
         return 0;
     }
     else
     {
+        (void) close (comm[1]);
+        (void) close (back_comm[0]);
         ctx->pid = pid;
         register_task_running (ctx, pid, comm[0], back_comm[1], info);
         return 1;
@@ -586,7 +593,7 @@ do_background (file_op_context_t * ctx, char *info)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-parent_call (void *routine, file_op_context_t * ctx, int argc, ...)
+parent_call (void *routine, file_op_context_t *ctx, int argc, ...)
 {
     int ret;
     va_list ap;
@@ -617,7 +624,7 @@ parent_call_string (void *routine, int argc, ...)
 
 /* event callback */
 gboolean
-background_parent_call (const gchar * event_group_name, const gchar * event_name,
+background_parent_call (const gchar *event_group_name, const gchar *event_name,
                         gpointer init_data, gpointer data)
 {
     ev_background_parent_call_t *event_data = (ev_background_parent_call_t *) data;
@@ -636,7 +643,7 @@ background_parent_call (const gchar * event_group_name, const gchar * event_name
 
 /* event callback */
 gboolean
-background_parent_call_string (const gchar * event_group_name, const gchar * event_name,
+background_parent_call_string (const gchar *event_group_name, const gchar *event_name,
                                gpointer init_data, gpointer data)
 {
     ev_background_parent_call_t *event_data = (ev_background_parent_call_t *) data;
